/*-*-linux-c-*-*/

/*
 *  gnewtellium - Newtella for Unix
 *  Copyright (C) 2001 Elias Athanasopoulos
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <string.h>
#include <dirent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdio.h>

#include <gtk/gtk.h>

#include "file.h"
#include "global.h"

#include "memory.h"

guint32 file_hashpjw (void *s, int len)
{
	/* taken from gnubile */
	const char *p;
	unsigned long int h;
	unsigned long topnibble;
	
	h = 479001599; /* start things with a very large prime. */
	
	for (p = (char*) s; len; p++, len--)
	{
		h = (h << 4) + *p;
		topnibble = h & 0xf0000000;     
		h = h ^ (topnibble >> 24) ^ topnibble;
	}
	return h;
}


int file_is_mp3(char *filename)
{
        return (!strncasecmp(filename+strlen(filename)-4, ".mp3", 4));               
}

guint32 file_get_size(char *file_name, char *path)
{
	char *s = newtella_malloc(1024);
	struct stat *fs = newtella_malloc(sizeof(struct stat));
	guint32 file_size;

	strncpy(s, path, 1024);
	s = strcat(s, "/");
	s = strcat(s, file_name);
       
	stat(s, fs);
	file_size = fs->st_size;

	newtella_free(fs);
	newtella_free(s);
	
	return file_size;
}

GList *file_extract_keywords(char *s)
{
	char *t, *key;
        GList *keywords = NULL;
                                       
        while((t = memchr(s, ' ', strlen(s)))) {
                *t++ = 0;
                key = newtella_malloc(255);
                strncpy(key, s, 255);
    
                /* valid keyword? */
                if (strlen(key) > 2)  
                        keywords = g_list_append(keywords, (char *) key);
		else 
			newtella_free(key);

		/* next keyword */
                strcpy(s, t);
        };
                
        /* last keyword */
        key = newtella_malloc(255);
        strncpy(key, s, 255);
         
        /* valid keyword? */
        if (strlen(key) > 2)
                keywords = g_list_append(keywords, (char *) key);
	else 
		newtella_free(key);

	return keywords;
}

int file_build_db(char *path)
{
	struct dirent *de;
        DIR *share_dir;
        struct file_entry *fe;

        share_dir = opendir(path);

        if (!share_dir) {
		perror("opendir");
		return -1;
	}

        while ((de = readdir(share_dir))) {
                if (strcmp(de->d_name, ".") && strcmp(de->d_name, "..") && file_is_mp3(de->d_name))
		{
			fe = newtella_malloc(sizeof(struct file_entry));
			fe->name = newtella_malloc(255);
			fe->path = newtella_malloc(255);
			strncpy(fe->name, de->d_name, 255);
			strncpy(fe->path, path, 255);
			fe->index = file_hashpjw(fe->name, 15);
			fe->size = file_get_size(fe->name, fe->path);
			gl_file_db = g_list_append(gl_file_db, fe);
		}
        }
        

	return 1;
}

int file_exclude_from_db(char *path)
{
	struct file_entry *fe;
	GList *iter;

	iter = g_list_first(gl_file_db);
	while (iter) {
		fe = iter->data;
		
		if (!strcmp(fe->path, path)) {
			gl_file_db = g_list_remove(gl_file_db, fe);
			newtella_free(fe);
			iter = g_list_first(gl_file_db);			
		} else
			iter = g_list_next(iter);
	}

	return 1;
}

GList *file_scan_db(char *key)
{
	GList *iter;
	struct file_entry *fe;
	GList *found_hits = NULL;

	iter = g_list_first(gl_file_db);
	while (iter) {
		fe = iter->data; 
		if (strstr(fe->name, key))
			found_hits = g_list_append(found_hits, fe);
		
		iter = g_list_next(iter);
	}

	return found_hits;
}









